import React, { useMemo, useState, useRef, forwardRef, useImperativeHandle } from 'react'
import { Table, Tooltip, Button, Popconfirm, Input, Space, useTable } from 'antd'
import { SearchOutlined } from '@ant-design/icons'
import ResizeableTitle from './components/ResizeableTitle'
import FileView from './components/FileView'
import CoordView from './components/CoordView'
import SelectView from './components/SelectView'
import IconPack from '../../../components/IconPack'
import { isObject } from '../../../../utils'

const getFileInfo = (formJson) => {
  let obj = {}
  let parent = formJson?.properties
  if (isObject(parent)) {
    for (let v in parent) {
      let child = parent[v]?.properties
      if (isObject(child)) {
        for (let s in child) {
          obj[s] = child[s]
        }
      }
    }
  }
  return obj
}

const TablePack = (props, ref) => {

  useImperativeHandle(ref, () => ({
    setFilteredInfo,
    setSortedInfo,
  }))

  const { parent, readOnly, config, fieldList, loading, dataSource, rowSelection, onRow, tableChange, btnsClick } = props
  const [fieldResize, setFieldResize] = useState({})
  const [filteredInfo, setFilteredInfo] = useState({})
  const [sortedInfo, setSortedInfo] = useState({})
  const inputRef = useRef()

  const getColumnSearchProps = (widgetInfo, dataIndex) => {
    const { widget, sourceType, options } = widgetInfo
    if (['CheckBox', 'ComboBox', 'RadioButton'].includes(widget)) {
      if (sourceType === '手动输入') {
        return {
          filters: options.map(v => { return { text: v.value, value: v.value, } }),
        }
      }
    }
    return {
      filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => {
        return (
          <div
            style={{ padding: 8 }}
            onKeyDown={(e) => e.stopPropagation()}
          >
            <Input
              ref={inputRef}
              placeholder={`搜索 ${dataIndex}`}
              value={selectedKeys[0]}
              onChange={e => {
                setSelectedKeys(e.target.value ? [e.target.value] : [])
              }}
              onPressEnter={() => {
                confirm({ closeDropdown: true })
              }}
              style={{
                marginBottom: 8,
                display: 'block',
                width: '280px',
              }}
            />
            <Space>
              <Button
                type="primary"
                onClick={() => {
                  confirm({ closeDropdown: true })
                }}
                icon={<SearchOutlined />}
                size="small"
                style={{ width: 90 }}
              >
                搜索
              </Button>
              <Button
                onClick={() => {
                  clearFilters()
                  confirm({ closeDropdown: true })
                }}
                size="small"
                style={{ width: 90 }}
              >
                重置
              </Button>
            </Space>
          </div>
        )
      },
      filterIcon: (filtered) => (
        <SearchOutlined
          style={{
            color: filtered ? '#1890ff' : undefined,
          }}
        />
      ),
    }
  }

  const btns = useMemo(() => {
    if (readOnly) {
      return [
        {
          title: '详情',
          icon: <IconPack.XiangQing />,
        },
      ]
    }
    return [
      {
        title: '详情',
        icon: <IconPack.XiangQing />,
      },
      {
        title: '编辑',
        icon: <IconPack.BianJi />,
      },
      {
        title: '删除',
        icon: <IconPack.ShanChu />,
      }
    ]
  }, [readOnly, parent])

  const fileColumns = useMemo(() => {
    const { webShowFieldGroup, formJson } = config
    let showField = []
    if (Array.isArray(fieldList) && fieldList.length > 0) {
      showField = fieldList
    } else {
      showField = webShowFieldGroup ? webShowFieldGroup.split(',').filter(v => v) : []
    }
    let json = getFileInfo(formJson)
    let array = []
    showField.forEach((v, i) => {
      const { widget } = json?.[v] || {}
      console.log('json', json, v)
      array.push({
        title: json[v]?.title || v,
        dataIndex: v,
        width: fieldResize[v] || 120,
        ellipsis: true,
        sorter: true,
        sortOrder: sortedInfo.columnKey === v ? sortedInfo.order : null,
        filteredValue: filteredInfo[v] || null,
        ...getColumnSearchProps(json[v], v),
        onHeaderCell: (column) => ({
          width: column.width,
          onResize: (e, props) => {
            setFieldResize({ ...fieldResize, [v]: props?.size?.width })
          },
        }),
        render: (value, r) => {
          let props = { ...json[v], value }
          if (widget === 'FileUpload') {
            return <FileView {...props} />
          }
          if (['Coordinate', 'Device', 'DrawPath', 'DrawArea'].includes(widget)) {
            return <CoordView {...props} />
          }
          if (['RadioButton', 'CheckBox'].includes(widget)) {
            return <SelectView {...props} />
          }
          return (
            <div>{value}</div>
          )
        }
      })
    })
    return array
  }, [config, fieldList, fieldResize, filteredInfo, sortedInfo])

  const columns = useMemo(() => {
    let columns = [
      {
        title: '序号',
        dataIndex: 'r',
        key: 'r',
        width: 50,
        align: 'center',
        fixed: 'left',
      },
      ...fileColumns,
      {
        title: '操作',
        width: btns.length * 35 + 30,
        align: 'center',
        fixed: 'right',
        render: (r) => {
          return (
            <div style={{ display: 'flex', justifyContent: 'center' }}>
              {
                btns.map(v => {
                  if (v.title === '删除') {
                    return (
                      <Popconfirm
                        key={v.title}
                        title='确定要删除数据吗？'
                        onConfirm={() => btnsClick(v.title, r.ID)}
                        okText='确定'
                        cancelText='取消'
                      >
                        <Button
                          type='link'
                          title='删除'
                          style={{ padding: '0 7px' }}
                          icon={v.icon}
                        />
                      </Popconfirm>
                    )
                  }
                  return (
                    <Button
                      key={v.title}
                      type='link'
                      title={v.title}
                      icon={v.icon}
                      style={{ padding: '0 7px' }}
                      onClick={() => btnsClick(v.title, r.ID)}
                    >
                    </Button>
                  )
                })
              }
            </div>
          )
        }
      }
    ]
    if (parent === '台账选择器') {
      return fileColumns
    }
    return columns
  }, [parent, btns, fileColumns])

  const onChange = (page, filters, sorter) => {
    setFilteredInfo(filters)
    setSortedInfo({ order: sorter.order, columnKey: sorter.field })
    tableChange(page, filters, sorter)
  }

  return (
    <Table
      size='small'
      rowKey='ID'
      bordered
      loading={loading}
      columns={columns}
      dataSource={dataSource}
      rowSelection={rowSelection}
      onRow={onRow}
      pagination={false}
      scroll={{ x: '100%', y: 'calc(100% - 40px)' }}
      onChange={onChange}
      components={{
        header: {
          cell: ResizeableTitle
        }
      }}
    />
  )

}

export default forwardRef(TablePack)